--Bar Class

--Control that contains a window as a border, interior shading and a bar.

--[[Bar.	Window			Object
			Background		Object
			Bar					Object
			Data.				x,y						Location
									sx,sy					Size
									BorderWidth		Width of Border
									Orient					Orientation l,r,u,d (fill direction - which edge does it empty towards?)]]--

Bar=class(Turbine.UI.Control)

function Bar:Constructor()
	
	self.Data={}
	
--Window is the basis for the two controls
	self.Window=Turbine.UI.Window()
	self.Window:SetVisible(true)
	self.Window:SetMouseVisible(false)
	
--Background Control
	self.Background=Turbine.UI.Control()
	self.Background:SetParent(self.Window)
	self.Background:SetVisible(true)
	self.Background:SetMouseVisible(false)
	
--Bar Control
	self.Bar=Turbine.UI.Control()
	self.Bar:SetParent(self.Window)
	self.Bar:SetVisible(true)
	self.Bar:SetMouseVisible(false)
	
	--[[self.Bar:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)	
	self.Bar:SetSize(175,25)
	self.Bar:SetBackground (0x4100886b)]]
	
--Bar Control for temporary morale/power - color is a similar purple color that is used in game for "bubble" morale/power, size is set to 0 to hide it til needed
	self.TempBar=Turbine.UI.Control()
	self.TempBar:SetParent(self.Window)
	self.TempBar:SetVisible(true)
	self.TempBar:SetMouseVisible(false)
	self.TempBar:SetBackColor(Turbine.UI.Color(1, 0.5, 0, 0.5))
	self.TempBar:SetSize(0,0)
	
	--[[self.TempBar:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)	
	self.TempBar:SetSize(175,25)
	self.TempBar:SetBackground (0x4100886b)]]
	
--TextBox
	self.Text=Turbine.UI.Label()
      self.Text:SetParent( self.Window )
      self.Text:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter )
	  self.Text:SetOutlineColor( Turbine.UI.Color(1, 0, 0, 0 ) )
	  self.Text:SetFont(Turbine.UI.Lotro.Font[Option.Font])
	  self.Text:SetFontStyle( Turbine.UI.FontStyle.Outline )
	  self.Text:SetMouseVisible(false)
	
--Dragbar including Rotator
	self.Window.DragBar = Deusdictum.UI.DragBar( self.Window,"Drag Bar")
	self.Window.DragBar:SetRotator( true )
	self.Window.DragBar:SetRotatorOffset(20, 20)
	self.Window.RotateStart = function () self:Rotate() end
	
	self.Window.MouseDown  =function(sender,args)
		if self.Window.DragBar:IsVisible() then
			Moving=true
		end
	end
	
	self.Window.MouseUp =function(sender,args)
		
		if Moving then 
			Moving=false
		end
		
	end
	
	self.Window.MouseMove = function(sender,args)
		if Moving then
			local x,y=self.Window:GetMousePosition()
			if x<5 then x=5 end
			if y<5 then y=5 end
			--If the alt key is down round the size to the nearest 5px
			if self.Window:IsAltKeyDown() and not(loading) then
				x=math.ceil(x/5)*5 y=math.ceil(y/5)*5
			end

			self:SetSize(x,y)
			
			self:SetFill(0.7, 0.8, 0.8) self:SetOpacity(1) 
			self.Window.DragBar:Refresh()
		end
	end
	
	self.Window.PositionChanged=function (sender,args)
		--If the alt key is being held down then snap to a background grid that is 16x16
		if self.Window:IsAltKeyDown() and not(loading) then
			local x,y=self.Window:GetPosition()
			x=math.ceil(x/16)*16 y=math.ceil(y/16)*16
			return self:SetPosition(x,y)
		end
	end
end

function Bar:SetMouseVisible(n)
	self.Window:SetMouseVisible(n)
end

function Bar:SetName(s)
	--Mainly just keyed into DragBar.
	self.Data.name=s
	self.Window.DragBar.sName=s
	self.Window.DragBar.Label:SetText( s )
end

function Bar:SetSize(sx,sy)
	self.Data.sx,self.Data.sy=sx,sy
	self.Window:SetSize(sx,sy)
	self.Background:SetSize(sx,sy)
	self:Refresh()
	self.Window.DragBar:Refresh()
	self.Text:SetSize(sx,sy)
end

function Bar:SetPosition(x,y)
	self.Window:SetPosition(x,y)
	self.Data.x,self.Data.y=x,y
	self.Window.DragBar:Refresh()
end

function Bar:SetText(s)
	self.Text:SetText( s)
end


function Bar:GetPosition()
	self.Data.x,self.Data.y=self.Window:GetPosition()
	return self.Data.x,self.Data.y
end

--function Bar:SetFill(n)
--	local o=self.Data.Orient or "l"
--	local width=self.Data.BorderWidth or 2
--	self.Data.Fill=n
--	if o=="l" or o=="r" then
--		x=self.Data.sx*n-2*width y=self.Data.sy-2*width
--	else
--		x=self.Data.sx-2*width y=self.Data.sy*n-2*width
--	end
--	
--	self.Bar:SetSize(x,y)
--	self:RefreshBar()
--	--Scaled opacity.
--		n=(1-n)*(1/Option.Opacity)
--		if Option.Opacity==0 then n=1 end
--		if n>1 then n=1 end
--		self:SetOpacity(n) 
--	--end
--end

-- Stever1388 patch to have SetFill that separates temp morale/power from regular morale/power
-- 	ntc - "N"ormal morale/power "T"o "C"ombined (temp+normal) morale/power fraction
-- 	t cur temp morale/power to max temp morale/power fraction
-- 	n cur morale/power to max morale/power fraction
function Bar:SetFill(ntc, t, n)
	local o=self.Data.Orient or "l"
	local width=self.Data.BorderWidth or 2
	--self.Data.Fill=ntc

	if o=="l" or o=="r" then
		if ntc < 1 then -- only need tempbar if there is some temp morale/power
			x=self.Data.sx*n*ntc-width y=self.Data.sy-2*width -- sets xsize and ysize of normal morale/power, note that width is only x1 for x
						
			xt=self.Data.sx*t*(1-ntc)-width yt=self.Data.sy-2*width -- sets xsize and ysize of temp morale/power

			-- setup temporary bar depending on orientation of bars
			if o=="l" then self.TempBar:SetPosition(self.Data.sx*ntc, width)
			else self.TempBar:SetPosition(self.Data.sx-(self.Data.sx*ntc)-xt, width) end

		else -- old code for when no temp morale/power is present (set size of tempbar to 0 to hide it)
			x=self.Data.sx*n-2*width y=self.Data.sy-2*width
			xt=0 yt=0
		end
	else
		if ntc < 1 then
			x=self.Data.sx-2*width y=self.Data.sy*n*ntc-width -- once again, note that width is only x1 for y
			
			xt=self.Data.sx-2*width yt=self.Data.sy*t*(1-ntc)-width

			-- setup temporary bar
			if o=="u" then self.TempBar:SetPosition(width, self.Data.sy*ntc)
			else self.TempBar:SetPosition(width, self.Data.sy-(self.Data.sy*ntc)-yt) end

		else -- old code for when no temp morale/power is present (set size of tempbar to 0 to hide it)
			x=self.Data.sx-2*width y=self.Data.sy*n-2*width
			xt=0 yt=0
		end
	end

	self.Bar:SetSize(x,y)
	self.TempBar:SetSize(xt,yt)
	self:RefreshBar()
	--Scaled opacity.
		n=(1-n)*(1/Option.Opacity)
		t=(1-t)*(1/Option.Opacity) -- gets the opacity setting for temp/maxtemp morale/power.
		if Option.Opacity==0 then n=1 end
		if n>1 or t>1 then n=1 -- if either are higher than 1, set opacity to 1
		elseif t>n then n=t end -- set opacity to which ever is highest
		self:SetOpacity(n) 
	--end
end

function Bar:RefreshBar()
	local o=self.Data.Orient or "l"
	local width=self.Data.BorderWidth or 2
	local x,y=self.Bar:GetSize()
	if 			o=="l" then
		self.Bar:SetPosition(width,width)
	elseif 	o=="r" then
		self.Bar:SetPosition(self.Window:GetWidth()-x-width,width)
	elseif	o=="u" then
		self.Bar:SetPosition(width,width)
	elseif	o=="d" then
		self.Bar:SetPosition(width,self.Window:GetHeight()-y-width)
	end
end

function Bar:Rotate()
	local o=self.Data.Orient or "l"
	local w,h=self.Window:GetSize()
	local bw,bh=self.Bar:GetSize()
	
	if 	o=="l" then
			o="r" 
	elseif 	o=="r" then
			o="u"	self:SetSize(h,w) self.Bar:SetSize(bh,bw)
	elseif	o=="u" then
			o="d"
	elseif	o=="d" then
			o="l"	self:SetSize(h,w)  self.Bar:SetSize(bh,bw)
	end
	self:Orient(o)
	self:Refresh()
end

function Bar:Orient(n)
	self.Data.Orient=n
	self:RefreshBar()
end

function Bar:SetBorderColor(n)
	self.Window:SetBackColor(n)
end

function Bar:SetBorderWidth(n)
	self.Data.BorderWidth=n or 2
	self:Refresh()
end

function Bar:Refresh()
	local width=self.Data.BorderWidth or 2
	self.Background:SetSize(self.Window:GetWidth()-2*width,self.Window:GetHeight()-2*width)
	self.Background:SetPosition(width,width)
	self:RefreshBar()
end
function Bar:SetBackgroundColor(n)
	self.Background:SetBackColor(n)
end
function Bar:SetBarColor(n)
	self.Bar:SetBackColor(n)
end
function Bar:SetOpacity(n)
	self.Window:SetOpacity(n)
end
function Bar:GetOpacity()
	return self.Window:GetOpacity()
end
